bool reset;
StructuredBuffer<float3> resetPos;
StructuredBuffer<float> radius;
StructuredBuffer<float> mass;
StructuredBuffer<float> dynamic;

struct particle
{
	float3 pos;
	float3 vel;
	float rad;
	float mass;
	float inversemass;
	float dynamic;
	float2 colide;
	float id;
};

RWStructuredBuffer<particle> Output : BACKBUFFER;
int ParticleCount;
float damp;
float3 BoxSyze;
float3 gravity;
float restitution;


//==============================================================================
//COMPUTE SHADER ===============================================================
//==============================================================================
 bool colliding(particle sphere,particle sphere2)
{
    float3 d = sphere.pos - sphere2.pos;
    float sumRadius = sphere.rad + sphere2.rad;
    float sqrRadius = sumRadius * sumRadius;
    float distSqr = (d.x * d.x) + (d.y * d.y) + (d.z * d.z);

    if (distSqr <= sqrRadius)
    {
        return true;
    }
    return false;
}
 particle resolveCollision(particle sphere,particle sphere2)
{
    // get the mtd
    float3 delta = sphere.pos-sphere2.pos;
    float d = length(delta);
    // minimum translation distance to push balls apart after intersecting
    float3 mtd = delta *(((sphere.rad + sphere2.rad)-d)/d); 
    // resolve intersection --
    // inverse mass quantities
    float TotalMass = sphere.inversemass + sphere2.inversemass;
    // push-pull them apart based off their mass
     sphere.pos = sphere.pos + (mtd * (sphere.inversemass / TotalMass));
    //ball.position = ball.position.subtract(mtd.multiply(im2 / (im1 + im2)));
    // impact speed
    float3 v = (sphere.vel - sphere2.vel);
    //float vn = v.dot(.normalize(mtd));
    float vn = dot(v,normalize(mtd));
    // sphere intersecting but moving away from each other already
    if (vn > 0.0f) {
    sphere.colide = float2(1,sphere2.id);
    return sphere;}
    // collision impulse
    float i = (-(1.0f + restitution) * vn) /TotalMass;
    float3 impulse = normalize(mtd)* (i);
    // change in momentum
	sphere.colide = float2(1,sphere2.id);
    sphere.vel +=  (impulse*(sphere.inversemass));
    return sphere;
}
[numthreads(128, 1, 1)]
void Repulsion( uint3 DTid : SV_DispatchThreadID )
{
	float r = Output[DTid.x].rad;
	Output[DTid.x].colide = float2(0.0,-1.0);
	  if(Output[DTid.x].dynamic==0){
			Output[DTid.x].pos = resetPos[DTid.x];
			Output[DTid.x].vel = 0;
		    Output[DTid.x].mass = mass[DTid.x];
		}
		if(Output[DTid.x].pos.y  > BoxSyze.y-r && Output[DTid.x].dynamic==1)
			{Output[DTid.x].pos.y = BoxSyze.y-r;
			Output[DTid.x].vel.y *=	 - restitution;
			}
	    else if(Output[DTid.x].pos.y < -BoxSyze.y+r&& Output[DTid.x].dynamic==1)
			{Output[DTid.x].pos.y = -BoxSyze.y+r;
				Output[DTid.x].vel.y *=	 - restitution;
			}
	    if(Output[DTid.x].pos.x > BoxSyze.x-r&& Output[DTid.x].dynamic==1) 
			{ Output[DTid.x].pos.x = BoxSyze.x-r;
				Output[DTid.x].vel.x *=	 - restitution;
			}
	    else if(Output[DTid.x].pos.x < -BoxSyze.x+r&& Output[DTid.x].dynamic==1) 
			{Output[DTid.x].pos.x = -BoxSyze.x+r;
				Output[DTid.x].vel.x *=	 - restitution;
			}
	    if(Output[DTid.x].pos.z > BoxSyze.z-r && Output[DTid.x].dynamic==1) 
			{Output[DTid.x].pos.z = BoxSyze.z-r;
			Output[DTid.x].vel.z *=	 - restitution;
			}
		else if(Output[DTid.x].pos.z < -BoxSyze.z+r && Output[DTid.x].dynamic==1)
			{Output[DTid.x].pos.z =-BoxSyze.z+r;
				Output[DTid.x].vel.z *=	 - restitution;
			}
	if (reset)
	{
		Output[DTid.x].pos = resetPos[DTid.x];
		Output[DTid.x].vel = 0;	
		Output[DTid.x].mass = mass[DTid.x];
		Output[DTid.x].inversemass = 1/mass[DTid.x];
		Output[DTid.x].dynamic = dynamic[DTid.x];
		Output[DTid.x].colide = float2(0.0,-1.0);
		Output[DTid.x].id = DTid.x;
	}
	else
	{
		for( uint i = 0; i < uint(ParticleCount); i++ )
		{
			if (DTid.x != i &&   Output[DTid.x].dynamic==1){
				if (colliding(Output[DTid.x],Output[i])){
				Output[DTid.x]= resolveCollision(Output[DTid.x],Output[i]);
				 }        
			}		
		}	
        Output[DTid.x].vel += gravity*0.0001;
		Output[DTid.x].pos+= Output[DTid.x].vel;
		Output[DTid.x].vel *= damp;
		Output[DTid.x].rad = radius[DTid.x];
		
	}				
}

//==============================================================================
//TECHNIQUES ===================================================================
//==============================================================================

technique11 simulation
{
	pass P0
	{
		SetComputeShader( CompileShader( cs_5_0, Repulsion() ) );
	}
}
